@@ -446,7 +446,8 @@ class WordType(caching.base.CachingMixin,models.Model):
     def getNDs(self):
         if self.ND is not None and self.FWND is not None:
             return self.ND,self.FWND
-        qs = WordType.objects.filter(Label__regex="$[^{<]").extra(
+        any_segment = '[A-Za-z]{1,2}'
+        qs = WordType.objects.filter(Label__regex="^[^{<]").extra(
                     select = {
                         'ur': """(SELECT GROUP_CONCAT(buckeyebrowser_segmenttype.Label SEPARATOR ' ')
                                 FROM buckeyebrowser_underlying
@@ -496,31 +497,48 @@ class WordType(caching.base.CachingMixin,models.Model):
         return self.SPhonoProb,self.BiPhonoProb
 
     def setViteLuceProb(self):
-        SPhoneProb = 0.0
-        BiPhoneProb = 0.0
-        ur = self.underlying_set.all()
-        for s in ur:
-            posq = WordType.objects.filter(underlying__Ordering=s.Ordering,underlying__SegmentType=s.SegmentType)
-            pos = 0.0
-            for pr in posq:
-                pos += pr.getFreq()
-            allposq = WordType.objects.filter(underlying__Ordering=s.Ordering)
-            allpos = 0.0
-            for pr in allposq:
-                allpos += pr.getFreq()
-            SPhoneProb += float(pos)/float(allpos)
-        if len(ur) > 1:
-            for i in xrange(len(ur)-1):
-                posq = WordType.objects.filter(underlying__Ordering=ur[i].Ordering,underlying__SegmentType=ur[i].SegmentType).filter(underlying__Ordering=ur[i+1].Ordering,underlying__SegmentType=ur[i+1].SegmentType)
-                pos = 0.0
-                for pr in posq:
-                    pos += pr.getFreq()
-                allposq = WordType.objects.filter(underlying__Ordering=ur[i].Ordering).filter(underlying__Ordering=ur[i+1].Ordering)
-                allpos = 0.0
-                for pr in allposq:
-                    allpos += pr.getFreq()
-                BiPhoneProb += float(pos)/float(allpos)
-        self.SPhonoProb,self.BiPhonoProb = SPhoneProb,BiPhoneProb
+        any_segment = '[A-Za-z]{1,2}'
+        qs = WordType.objects.filter(Label__regex="^[^{<]").extra(
+                    select = {
+                        'ur': """(SELECT GROUP_CONCAT(buckeyebrowser_segmenttype.Label SEPARATOR ' ')
+                                FROM buckeyebrowser_underlying
+                                INNER JOIN buckeyebrowser_segmenttype
+                                ON buckeyebrowser_segmenttype.id
+                                    = buckeyebrowser_underlying.SegmentType_id
+                                WHERE buckeyebrowser_underlying.WordType_id
+                                    =buckeyebrowser_wordtype.id)"""})
+        word = qs.get(pk = self.pk)
+        patterns = []
+        SPprob = 0.0
+        BPprob = 0.0
+        phones = input_string.split(" ")
+        for i in range(len(phones)):
+            patt = [any_segment] * i
+            patt.append(phones[i])
+            pattern = '^'+' '.join(patt) +'.*$'
+            totPattern = '^'+' '.join([any_segment] * (i+1)) +'.*$'
+            if no_stress:
+                count = len(PhonoString.objects.filter(NoStress__regex = pattern))
+                totCount = len(PhonoString.objects.filter(NoStress__regex = totPattern))
+            else:
+                count = len(PhonoString.objects.filter(Transcription__regex = pattern))
+                totCount = len(PhonoString.objects.filter(Transcription__regex = totPattern))
+            SPprob += float(count) / float(totCount)
+            if i != len(phones)-1:
+                patt = [any_segment] * i
+                patt.extend([phones[i],phones[i+1]])
+                pattern = '^'+' '.join(patt) +'.*$'
+                totPattern = '^'+' '.join([any_segment] * (i+2)) +'.*$'
+                if no_stress:
+                    count = len(PhonoString.objects.filter(NoStress__regex = pattern))
+                    totCount = len(PhonoString.objects.filter(NoStress__regex = totPattern))
+                else:
+                    count = len(PhonoString.objects.filter(Transcription__regex = pattern))
+                    totCount = len(PhonoString.objects.filter(Transcription__regex = totPattern))
+                BPprob += float(count) / float(totCount)
+        SPprob = SPprob / float(len(phones))
+        BPprob = BPprob / float(len(phones)-1)
+        self.SPhonoProb,self.BiPhonoProb = SPprob,BPprob
         self.save()
 
 
